/* $Id: network.h,v 1.2 2000/10/23 04:42:18 bind Exp $ */

struct ip_header {
#if BYTE_ORDER == LITTLE_ENDIAN
  u_int8_t ip_hl:4,		/* header length */
      ip_v:4;			/* version */
#endif
#if BYTE_ORDER == BIG_ENDIAN
  u_int8_t ip_v:4,		/* version */
      ip_hl:4;			/* header length */
#endif
  u_int8_t ip_tos;		/* type of service */
  u_int16_t ip_len;		/* total length */
  u_int16_t ip_id;		/* identification */
  u_int16_t ip_off;		/* fragment offset field */
#define	IP_RF 0x8000		/* reserved fragment flag */
#define	IP_DF 0x4000		/* dont fragment flag */
#define	IP_MF 0x2000		/* more fragments flag */
#define	IP_OFFMASK 0x1fff	/* mask for fragmenting bits */
  u_int8_t ip_ttl;		/* time to live */
  u_int8_t ip_p;		/* protocol */
  u_int16_t ip_sum;		/* checksum */
  struct in_addr ip_src, ip_dst;/* source and dest address */
};

struct icmp_header {
  u_int8_t icmp_type;		/* type of message, see below */
  u_int8_t icmp_code;		/* type sub code */
  u_int16_t icmp_cksum;		/* ones complement cksum of struct */
  union {
    struct echo {
      n_short id;
      n_short seq;
    }   idseq;
  }   icmp_hun;
};

struct arp_header {
  u_int16_t ar_hrd;		/* format of hardware address */
  u_int16_t ar_pro;		/* format of protocol address */
  u_int8_t ar_hln;		/* length of hardware address */
  u_int8_t ar_pln;		/* length of protocol address */
  u_int16_t ar_op;		/* one of: */
  u_int8_t ar_sha[6];		/* sender hardware address */
  u_int8_t ar_spa[6];		/* sender protocol address */
  u_int8_t ar_tha[6];		/* target hardware address */
  u_int8_t ar_tpa[6];		/* target protocol address */
};

//typedef u_int32_t tcp_seq;

struct tcp_header {
  u_int16_t th_sport;		/* source port */
  u_int16_t th_dport;		/* destination port */
  tcp_seq th_seq;		/* sequence number */
  tcp_seq th_ack;		/* acknowledgement number */
#if BYTE_ORDER == LITTLE_ENDIAN
  u_int8_t th_x2:4,		/* (unused) */
      th_off:4;			/* data offset */
#endif
#if BYTE_ORDER == BIG_ENDIAN
  u_int8_t th_off:4,		/* data offset */
      th_x2:4;			/* (unused) */
#endif
  u_int8_t th_flags;
#define	TH_FIN	  0x01
#define	TH_SYN	  0x02
#define	TH_RST	  0x04
#define	TH_PUSH	  0x08
#define	TH_ACK	  0x10
#define	TH_URG	  0x20
  u_int16_t th_win;		/* window */
  u_int16_t th_sum;		/* checksum */
  u_int16_t th_urp;		/* urgent pointer */
};

struct udp_header {
  u_int16_t uh_sport;		/* source port */
  u_int16_t uh_dport;		/* destination port */
  u_int16_t uh_ulen;		/* udp length */
  u_int16_t uh_sum;		/* udp checksum */
};

struct dns_header {
  unsigned short int id;
  unsigned char rd:1;		/* recursion desired */
  unsigned char tc:1;		/* truncated message */
  unsigned char aa:1;		/* authoritive answer */
  unsigned char opcode:1;	/* purpose of message */
  unsigned char qr:1;		/* response flag */
  unsigned char rcode:1;	/* response code */
  unsigned char unused:1;	/* unused bits */
  unsigned char pr:1;		/* primary server required (non standard) */
  unsigned char ra:1;		/* recursion available */
  unsigned short int que_num;
  unsigned short int rep_num;
  unsigned short int num_rr;
  unsigned short int num_rrsup;
};
