/* $Id: net.c,v 1.3 2000/10/23 04:42:18 bind Exp $ */

#include <sentinel.h>
#include <network.h>

pthread_mutex_t global_mutex = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t global_cond = PTHREAD_COND_INITIALIZER;

extern pthread_cond_t finish_cond;

u_char *pkt;

/*
 * Function: sniff_pkt()
 * Purpose: sniff a packet off a selected interface,
 *          signal the processing thread.
 */

void *
sniff_pkt(void)
{
  pcap_t *socket;
  struct pcap_pkthdr pcap_h;

  pthread_mutex_lock(&global_mutex);
  socket = pcap_open_live(device, 1000, 1, 10, errbuf);
  pthread_mutex_unlock(&global_mutex);

  if (socket == NULL) {
    perror("pcap_open_live");
    exit(-1);
  }
  while (1) {
    pthread_mutex_lock(&global_mutex);
    pkt = (u_char *) pcap_next(socket, &pcap_h);

    if (pkt != NULL) {
      pthread_mutex_unlock(&global_mutex);
      pthread_cond_signal(&global_cond);
    }
    usleep(10);
  }
}

/*
 * Function: process_arp()
 * Purpose: Process captured arp packets.
 */

void *
process_arp(void)
{
  struct arp_header *arp;

  while (1) {
    pthread_mutex_lock(&global_mutex);
    pthread_cond_wait(&global_cond, &global_mutex);

    arp = (struct arp_header *) (pkt + 14);

    if (ntohs(arp->ar_op) == 2)
      if (memcmp(&(arp->ar_spa), &dst, 4) == 0) {
	printf("\nPositive to ARP test: %s\n", libnet_host_lookup(dst, 0));
	pthread_cond_signal(&finish_cond);
      }
    pthread_mutex_unlock(&global_mutex);
  }

}

/*
 * Function: send_arp()
 * Purpose: Send out a crafted arp packet.
 */

void *
send_arp(void *shit)
{
  u_char *pkt;			/* our outgoing packet */
  struct libnet_link_int *link;	/* link layer information */
  struct ether_addr *hwaddr;
  u_long src;
  pthread_mutex_t tmp = PTHREAD_MUTEX_INITIALIZER;
  u_char fakeaddr[6] = {0xff, 0x0, 0x0, 0x0, 0x0, 0x0};
  u_char defaddr[6] = {0x0, 0x0, 0x0, 0x0, 0x0, 0x0};
  char *dev, *source;
  int count, i;

  count = i = 0;
  dev = (char *) shit;

  pthread_mutex_lock(&tmp);
  count = num;
  pthread_mutex_unlock(&tmp);

  /* open link interface */
  if (!(link = libnet_open_link_interface(dev, errbuf))) {
    perror("libnet_link_init");
    exit(-1);
  }
  hwaddr = libnet_get_hwaddr(link, dev, errbuf);
  src = libnet_get_ipaddr(link, dev, errbuf);

  source = libnet_host_lookup(htonl(src), 0);
  src = libnet_name_resolve(source, 0);

  /* malloc memory for arp packet */
  if (libnet_init_packet(LIBNET_ARP_H + LIBNET_ETH_H, &pkt) < 0) {
    perror("libnet_init_packet");
    exit(1);
  }
  /* build ethernet header */
  libnet_build_ethernet(fakeaddr, fakeaddr, ETHERTYPE_ARP, NULL, 0, pkt);

  /* build arp header */
  libnet_build_arp(ARPHRD_ETHER, ETHERTYPE_IP, 6, 4, ARPOP_REQUEST,
	     (u_char *) hwaddr, (u_char *) & src, defaddr, (u_char *) & dst,
		   NULL, 0, pkt + ETH_H);

  printf("Sending out %d ARP packets", count);
  fflush(stdout);

  while (1) {
    i++;
    printf(".");
    fflush(stdout);
    /* write packet on layer 2 */
    libnet_write_link_layer(link, dev, pkt, LIBNET_ARP_H + LIBNET_ETH_H);
    sleep(1);			/* send one packet per second */
    if (i == count)
      break;
  }

  libnet_destroy_packet(&pkt);	/* free memory from packet */
  libnet_close_link_interface(link);	/* close interface */

  putchar('\n');
  pthread_cond_signal(&finish_cond);
  pthread_exit(0);
}

/*
 * Function: process_icmp()
 * Purpose: Process captured icmp packets.
 */

void *
process_icmp(void)
{
  struct ip_header *ip;
  struct icmp_header *icmp;
  char *target;

  while (1) {
    pthread_mutex_lock(&global_mutex);
    pthread_cond_wait(&global_cond, &global_mutex);

    ip = (struct ip_header *) (pkt + 14);
    icmp = (struct icmp_header *) (pkt + 14 + 20);

    target = inet_ntoa(ip->ip_src);	/* need to write threadsafe later */

    if (ip->ip_p != 1)
      continue;

    if (icmp->icmp_type == 0) {
      if (strstr(target, libnet_host_lookup(dst, 0)) != NULL)
	if (ntohs(icmp->icmp_hun.idseq.id) == 31337) {
	  printf("\nPositive to Etherping test: %s\n",
		 libnet_host_lookup(dst, 0));
	  pthread_cond_signal(&finish_cond);
	}
      pthread_mutex_unlock(&global_mutex);
    }
  }
}

/*
 * Function: send_etherping()
 * Purpose: Sends a ICMP echo packet with a bogus dest MAC addr.
 */

void *
send_etherping(void *shit)
{
  u_char *etherpkt;
  struct libnet_link_int *link;
  struct ether_addr *hwaddr;
  pthread_mutex_t tmp = PTHREAD_MUTEX_INITIALIZER;
  u_long src;
  char *source, *dev;
  int count, i = 0;
  u_char fakeaddr[6] = {0xff, 0x0, 0x0, 0x0, 0x0, 0x0};

  pthread_mutex_lock(&tmp);
  count = num;
  pthread_mutex_unlock(&tmp);

  dev = (char *) shit;

  link = libnet_open_link_interface(dev, errbuf);

  hwaddr = libnet_get_hwaddr(link, dev, errbuf);
  src = libnet_get_ipaddr(link, dev, errbuf);

  source = libnet_host_lookup(htonl(src), 0);
  src = libnet_name_resolve(source, 0);

  if (!link) {
    fprintf(stderr, "libnet_open_link_interface: %s\n", errbuf);
    exit(1);
  }
  if (libnet_init_packet(LIBNET_ICMP_ECHO_H + LIBNET_ETH_H +
			 LIBNET_IP_H, &etherpkt) < 0) {
    perror("libnet_init_packet");
    exit(1);
  }
  libnet_build_ethernet(fakeaddr, (u_char *) hwaddr, ETHERTYPE_IP, NULL, 0,
			etherpkt);

  libnet_build_ip(LIBNET_ICMP_ECHO_H, 0, 100, 0, 64, IPPROTO_ICMP,
		  src, dst, NULL, 0, etherpkt + LIBNET_ETH_H);

  libnet_build_icmp_echo(ICMP_ECHO, 0, 0x7a69, 1, NULL, 0, etherpkt +
			 LIBNET_IP_H + LIBNET_ETH_H);

  libnet_do_checksum(etherpkt + LIBNET_ETH_H, IPPROTO_IP, LIBNET_IP_H);
  libnet_do_checksum(etherpkt + LIBNET_ETH_H, IPPROTO_ICMP, LIBNET_ICMP_ECHO_H);

  printf("Sending out %d fake ICMP echo packets", num);

  while (1) {
    i++;
    printf(".");
    fflush(stdout);
    sleep(1);

    libnet_write_link_layer(link, dev, etherpkt, LIBNET_ICMP_H +
			    LIBNET_IP_H + LIBNET_ETH_H + 4);

    if (i == count)
      break;
  }

  libnet_close_link_interface(link);
  libnet_destroy_packet(&etherpkt);
  putchar('\n');
  pthread_cond_signal(&finish_cond);
  pthread_exit(0);
}

/*
 * Function: process_dns()
 * Purpose: Process DNS packets and look for match.
 */

void *
process_dns(void)
{
  struct ip_header *ip;
  struct udp_header *udp;
  struct dns_header *dns;
  char *data, *host;
  char names[100];
  int count;

  while (1) {
    pthread_mutex_lock(&global_mutex);
    pthread_cond_wait(&global_cond, &global_mutex);

    ip = (struct ip_header *) (pkt + 14);
    udp = (struct udp_header *) (pkt + 14 + 20);
    dns = (struct dns_header *) (pkt + 14 + 20 + 8);
    data = (char *) (pkt + 14 + 20 + 8 + 12);

    memset(names, 0x0, 100);

    if (ip->ip_p == 17) {
      if (ntohs(udp->uh_dport) == 53) {
	host = inet_ntoa(ip->ip_src);
	if (strstr(host, (char *) libnet_host_lookup(dst, 0)) == NULL)
	  break;
	if (!dns->que_num)
	  break;
	count = (char) *data;
	while (count) {
	  (char *) data++;
	  strncat(names, (char *) data, count);
	  data += count;
	  count = (char) *data;
	  strncat(names, ".", sizeof(names) - strlen(names));
	}

	names[strlen(names) - 1] = '\0';

	if (strstr(names, "123.123.123.123.in-addr.arpa") != NULL) {
	  printf("\nPositive to DNS test: %s\n", libnet_host_lookup(dst, 0));
	  pthread_cond_signal(&finish_cond);
	}
      }
    }
    pthread_mutex_unlock(&global_mutex);
  }
  return NULL;
  //stop bitching fuck face.
}

/*
 * Function: tcp_flood()
 * Purpose: Create fake tcp connections on a subnet.
 */

void *
tcp_flood(void *arg)
{
  struct libnet_link_int *link;
  char *pkts[5], *dev;
  u_long fake = libnet_name_resolve("123.123.123.123", 0);
  int len[5], j, n, count;
  struct ether_addr *hwaddr;
  pthread_mutex_t tmp = PTHREAD_MUTEX_INITIALIZER;
  j = n;

  dev = (char *) arg;

  pthread_mutex_lock(&tmp);
  count = num;
  pthread_mutex_unlock(&tmp);

  link = libnet_open_link_interface(dev, errbuf);

  if (!link) {
    perror("libnet_open_link_interface");
    exit(-1);
  }
  hwaddr = libnet_get_hwaddr(link, dev, errbuf);

  /* Craft our magical packets. */

  pkts[1] = tcp_craft(dst, fake, 12345, 0, TH_SYN, &len[1], 0, hwaddr);
  pkts[2] = tcp_craft(fake, dst, 98765, 12345, (TH_SYN | TH_ACK), &len[2], 0, hwaddr);
  pkts[3] = tcp_craft(dst, fake, 12346, 98766, TH_ACK, &len[3], 0, hwaddr);
  pkts[4] = tcp_craft(dst, fake, 12347, 98768, TH_ACK, &len[4], 1, hwaddr);
  pkts[5] = tcp_craft(dst, fake, 12348, 98768, TH_RST, &len[5], 0, hwaddr);

  printf("Creating %d fake TCP connections", count);

  for (j = 0; j <= count; j++) {
    printf(".");
    fflush(stdout);
    sleep(1);

    for (n = 1; n <= 5; n++)
      libnet_write_link_layer(link, dev, pkts[n], len[n]);
  }

  libnet_close_link_interface(link);

  putchar('\n');
  pthread_cond_signal(&finish_cond);
  pthread_exit(0);
}

/*
 * Function: tcp_craft()
 * Purpose: Craft a custom tcp packet.
 */

char *
tcp_craft(u_long source, u_long dest, int seq, int ack, u_char flags, int *len, int pf, struct ether_addr * hwaddr)
{
  u_char *packet, *pay = NULL, defether[6] = {0x22, 0x22, 0x22, 0x22, 0x22, 0x22};
  int payload_s = 0, port = 23;

  if (pf == 1) {
    pay = "SNIFFER FOUND";
    payload_s = strlen(pay);
  }
  libnet_init_packet(LIBNET_TCP_H + LIBNET_IP_H + LIBNET_ETH_H + payload_s, &packet);
  libnet_build_ethernet(defether, (u_char *) hwaddr, ETHERTYPE_IP, NULL, 0, packet);

  libnet_build_ip(LIBNET_TCP_H + payload_s, 0, 100, 0, 64, IPPROTO_TCP, source, dest, NULL, 0, packet + LIBNET_ETH_H);

  libnet_build_tcp(23, port, seq, ack, flags, 0, 0, pay, payload_s, packet + LIBNET_IP_H + LIBNET_ETH_H);

  libnet_do_checksum(packet + LIBNET_ETH_H, IPPROTO_IP, LIBNET_IP_H);
  libnet_do_checksum(packet + LIBNET_ETH_H, IPPROTO_TCP, LIBNET_TCP_H + payload_s);

  *len = LIBNET_TCP_H + LIBNET_IP_H + LIBNET_ETH_H + payload_s;
  return (packet);
}
