/* $Id: sentinel.c,v 1.2 2000/10/23 04:42:18 bind Exp $ */

/*
 * The Sentinel Project
 * Copyright (c) 1999-2000, Subterrain Security Group
 *
 * Written by: bind <bind@subterrain.net>
 */

#include <sentinel.h>

void banner(void);
void usage(char *);

pthread_mutex_t finish_mutex = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t finish_cond = PTHREAD_COND_INITIALIZER;

int 
main(int argc, char **argv)
{
  int opt, options = 0;
  extern char *optarg;
  extern int opterr;
  struct utsname hinfo;

  pthread_t threads[6];

  if (geteuid()) {
    printf("User '%s' needs euid 0\n", getlogin());
    exit(-1);
  }
  banner();

  if (argc < 2)
    usage(argv[0]);

  opterr = 0;
  while ((opt = getopt(argc, argv, "leado:i:n:t:")) != EOF) {
    switch (opt) {
      case 'l':
	options |= LATENCY;
	break;
      case 'e':
	options |= ETHERPING;
	break;
      case 'a':
	options |= ARP;
	break;
      case 'd':
	options |= DNS;
	break;
      case 'o':
	logfile = optarg;
	break;
      case 'i':
	device = optarg;
	break;
      case 'n':
	num = atoi(optarg);
	if (num == 0)
	  usage(argv[0]);
	break;
      case 't':
	dst = libnet_name_resolve(optarg, 1);
	break;
      case 'h':
      default:
	usage(argv[0]);
	break;
    }
  }

  if (num == 0)
    num = 10;

  uname(&hinfo);

  printf("\nRunning on: '%s' running %s %s on a(n) %s\n\n",
    hinfo.nodename, hinfo.sysname, hinfo.release, hinfo.machine);

  if (device == NULL)
    device = pcap_lookupdev(errbuf);
  if (device == NULL) {
    printf("Error: Unable to open device.\n");
    exit(-1);
  }
  if (dst <= 0)
    usage(argv[0]);

  if (options == 0) {
    printf("Must choose at least one method.\n");
    exit(-1);
  }
  printf("Device: %s\n", device);
  printf("Target: %s\n\n", libnet_host_lookup(dst, 0));

  if (options & ARP) {
    pthread_create(&threads[0], NULL, (void *) sniff_pkt, NULL);
    pthread_create(&threads[1], NULL, (void *) process_arp, NULL);
    pthread_create(&threads[2], NULL, (void *) send_arp, (void *) device);

    pthread_mutex_lock(&finish_mutex);
    pthread_cond_wait(&finish_cond, &finish_mutex);
    pthread_mutex_unlock(&finish_mutex);

    return 0;
  }
  if (options & ETHERPING) {
    pthread_create(&threads[0], NULL, (void *) sniff_pkt, NULL);
    pthread_create(&threads[1], NULL, (void *) process_icmp, NULL);
    pthread_create(&threads[2], NULL, (void *) send_etherping, (void *) device);

    pthread_mutex_lock(&finish_mutex);
    pthread_cond_wait(&finish_cond, &finish_mutex);
    pthread_mutex_unlock(&finish_mutex);

    return 0;
  }
  if (options & DNS) {
    pthread_create(&threads[0], NULL, (void *) sniff_pkt, NULL);
    pthread_create(&threads[1], NULL, (void *) process_dns, NULL);
    pthread_create(&threads[2], NULL, (void *) tcp_flood, (void *) device);
    pthread_mutex_lock(&finish_mutex);
    pthread_cond_wait(&finish_cond, &finish_mutex);
    pthread_mutex_unlock(&finish_mutex);

    return 0;
  }
  return 0;
}

/*
 * Function: banner()
 * Purpose: Print out program name and author in center of screen.
 */

void 
banner()
{
  printf("\n%s (%s)\n"
    "Copyright (c) 1999-2000 Subterrain Security Group\n"
    "Written by bind@subterrain.net\n\n", TITLE, VERSION);
}

/*
 * Function: usage()
 * Purpose: Display command line usage.
 */

void 
usage(char *arg)
{
  printf("Usage:\n"
    "  %s [method] [options] [-t <target>]\n\n"
    "Methods:\n"
    "  [ -l ICMP Ping Latency test ]\n"
    "  [ -e Etherping test ]\n"
    "  [ -a ARP test ]\n"
    "  [ -d DNS test ]\n\n"
    "Options:\n"
    "  [ -n Number of packets to send ]\n"
    "  [ -i Set default interface ]\n\n", arg);

  exit(-1);
}
